#ifndef ARENSTORF
#define ARENSTORF

template<class T, class N=T>
class ArenstorfOrbitModel
{
public:
  /** \brief export size_type */
  typedef std::size_t size_type;

  /** \brief export time_type */
  typedef T time_type;

  /** \brief export number_type */
  typedef N number_type;

  //! constructor stores parameter lambda
  ArenstorfOrbitModel (): mu2(0.012277471), mu1(1.0-mu2), counter(0)
  {}

  //! return number of componentes for the model
  std::size_t size () const
  {
    return 4;
  }

  //! set initial state including time value
  void initialize (T& t0, hdnum::Vector<N>& x0) const
  {
    t0 = 0;
    x0[0] = 0.994;
    x0[1] = 0;
    x0[2] = 0;
    x0[3] = -2.0015851063790825224;
}

  //! model evaluation
  void f (const T& t, const hdnum::Vector<N>& x, hdnum::Vector<N>& result) const
  {
    T N1 = (x[0]+mu2)*(x[0]+mu2)+x[1]*x[1];
    N1=N1*N1*N1;
    N1=std::sqrt(N1);
    T N2 = (x[0]-mu1)*(x[0]-mu1)+x[1]*x[1];
    N2=N2*N2*N2;
    N2=std::sqrt(N2);
    result[0] = x[2];
    result[1] = x[3];
    result[2] = x[0] + 2*x[3] - mu1*(x[0]+mu2)/N1 - mu2*(x[0]-mu1)/N2;
    result[3] = x[1] - 2*x[2] - mu1*x[1]/N1 - mu2*x[1]/N2;
    counter++;
  }

  //! jacobian evaluation needed for implicit solvers
  // void f_x (const T& t, const Vector<N>& x, DenseMatrix<N>& result) const
  // {
  // }

  void reset_counter() const
  {
    counter = 0;
  }

  int f_evaluations() const
  {
    return counter;
  }

private:
  const N mu2;
  const N mu1;
  mutable int counter;
};

#endif
